@@ -671,14 +671,13 @@ int mount_autofs_offset(struct autofs_point *ap, struct mapent *me, const char *
 		if (!(ret == -1 && errno == ENOENT))
 			return MOUNT_OFFSET_FAIL;
 	} else {
-/*
 		if (is_mounted(_PROC_MOUNTS, me->key, MNTS_AUTOFS)) {
 			if (ap->state != ST_READMAP)
 				warn(ap->logopt,
 				     "trigger %s already mounted", me->key);
 			return MOUNT_OFFSET_OK;
 		}
-*/
+
 		if (me->ioctlfd != -1) {
 			error(ap->logopt, "active offset mount %s", me->key);
 			return MOUNT_OFFSET_FAIL;
@@ -913,7 +912,7 @@ void *expire_proc_direct(void *arg)
 			cache_writelock(me->mc);
 			if (me->ioctlfd != -1 && 
 			    fstat(me->ioctlfd, &st) != -1 &&
-			    !count_mounts(ap->logopt, next->path, st.st_dev)) {
+			    !count_mounts(ap, next->path, st.st_dev)) {
 				ops->close(ap->logopt, me->ioctlfd);
 				me->ioctlfd = -1;
 				cache_unlock(me->mc);
@@ -1385,7 +1384,7 @@ int handle_packet_missing_direct(struct autofs_point *ap, autofs_packet_missing_
 		master_source_unlock(ap->entry);
 		master_mutex_unlock();
 		pthread_setcancelstate(state, NULL);
-		return 1;
+		return 0;
 	}
 
 	/* Check if we recorded a mount fail for this key */
@@ -1410,7 +1409,7 @@ int handle_packet_missing_direct(struct autofs_point *ap, autofs_packet_missing_
 		master_source_unlock(ap->entry);
 		master_mutex_unlock();
 		pthread_setcancelstate(state, NULL);
-		return 1;
+		return 0;
 	}
 
 	mt = malloc(sizeof(struct pending_args));
@@ -1424,7 +1423,7 @@ int handle_packet_missing_direct(struct autofs_point *ap, autofs_packet_missing_
 		master_source_unlock(ap->entry);
 		master_mutex_unlock();
 		pthread_setcancelstate(state, NULL);
-		return 1;
+		return 0;
 	}
 	memset(mt, 0, sizeof(struct pending_args));
 
