@@ -100,25 +100,64 @@ static int do_read_master(struct master *master, char *type, time_t age)
 	return status;
 }
 
-static char *find_map_path(struct map_source *map)
+static char *find_map_path(struct autofs_point *ap, struct map_source *map)
 {
+	const char *mname = map->argv[0];
+	unsigned int mlen = strlen(mname);
+	char *tok, *ptr = NULL;
+	char *path = NULL;
+	char *search_path;
 	struct stat st;
-	char *path;
 
-	path = malloc(strlen(AUTOFS_MAP_DIR) + strlen(map->argv[0]) + 2);
-	if (!path)
+	/*
+	 * This is different to the way it is in amd.
+	 * autofs will always try to locate maps in AUTOFS_MAP_DIR
+	 * but amd has no default and will not find a file map that
+	 * isn't a full path when no search_path is configured, either
+	 * in the mount point or global configuration.
+	 */
+	search_path = strdup(AUTOFS_MAP_DIR);
+	if (map->flags & MAP_FLAG_FORMAT_AMD) {
+		struct autofs_point *pap = ap;
+		char *tmp;
+		/*
+		 * Make sure we get search_path from the root of the
+		 * mount tree, if one is present in the configuration.
+		 * Again different from amd, which ignores the submount
+		 * case.
+		 */
+		while (pap->parent)
+			pap = pap->parent;
+		tmp = conf_amd_get_search_path(pap->path);
+		if (tmp) {
+			if (search_path)
+				free(search_path);
+			search_path = tmp;
+		}
+	}
+	if (!search_path)
 		return NULL;
 
-	strcpy(path, AUTOFS_MAP_DIR);
-	strcat(path, "/");
-	strcat(path, map->argv[0]);
+	tok = strtok_r(search_path, ":", &ptr);
+	while (tok) {
+		char *this = malloc(strlen(tok) + mlen + 2);
+		if (!this) {
+			free(search_path);
+			return NULL;
+		}
+		strcpy(this, tok);
+		strcat(this, "/");
+		strcat(this, mname);
+		if (!stat(this, &st)) {
+			path = this;
+			break;
+		}
+		free(this);
+		tok = strtok_r(NULL, ":", &ptr);
+	}
 
-	if (!stat(path, &st))
-		return path;
-
-	free(path);
-
-	return NULL;
+	free(search_path);
+	return path;
 }
 
 static int read_master_map(struct master *master, char *type, time_t age)
@@ -194,8 +233,8 @@ int lookup_nss_read_master(struct master *master, time_t age)
 			memset(source, 0, 10);
 			if ((!strncmp(name, "file", 4) &&
 				 (name[4] == ',' || name[4] == ':')) ||
-			    (!strncmp(name, "yp", 3) &&
-				 (name[3] == ',' || name[3] == ':')) ||
+			    (!strncmp(name, "yp", 2) &&
+				 (name[2] == ',' || name[2] == ':')) ||
 			    (!strncmp(name, "nis", 3) &&
 				 (name[3] == ',' || name[3] == ':')) ||
 			    (!strncmp(name, "nisplus", 7) &&
@@ -435,7 +474,7 @@ static int lookup_map_read_map(struct autofs_point *ap,
 	if (map->argv[0][0] == '/')
 		return do_read_map(ap, map, age);
 
-	path = find_map_path(map);
+	path = find_map_path(ap, map);
 	if (!path)
 		return NSS_STATUS_UNKNOWN;
 
@@ -476,8 +515,10 @@ static enum nsswitch_status read_map_source(struct nss_source *this,
 	}
 
 	this->source[4] = '\0';
+	tmap.flags = map->flags;
 	tmap.type = this->source;
 	tmap.format = map->format;
+	tmap.name = map->name;
 	tmap.lookup = map->lookup;
 	tmap.mc = map->mc;
 	tmap.instance = map->instance;
@@ -488,7 +529,7 @@ static enum nsswitch_status read_map_source(struct nss_source *this,
 	tmap.argc = 0;
 	tmap.argv = NULL;
 
-	path = find_map_path(map);
+	path = find_map_path(ap, map);
 	if (!path)
 		return NSS_STATUS_UNKNOWN;
 
@@ -595,6 +636,13 @@ int lookup_nss_read_map(struct autofs_point *ap, struct map_source *source, time
 		list_for_each(p, head) {
 			this = list_entry(p, struct nss_source, list);
 
+			if (map->flags & MAP_FLAG_FORMAT_AMD &&
+			    !strcmp(this->source, "sss")) {
+				warn(ap->logopt,
+				     "source sss is not available for amd maps.");
+				continue;
+			}
+
 			debug(ap->logopt,
 			      "reading map %s %s", this->source, map->argv[0]);
 
@@ -668,6 +716,14 @@ int lookup_ghost(struct autofs_point *ap, const char *root)
 		cache_readlock(mc);
 		me = cache_enumerate(mc, NULL);
 		while (me) {
+			/*
+			 * Map entries that have been created in the cache
+			 * due to a negative lookup shouldn't have directories
+			 * created if they haven't already been created.
+			 */
+			if (!me->mapent)
+				goto next;
+
 			if (!strcmp(me->key, "*"))
 				goto next;
 
@@ -746,6 +802,74 @@ int do_lookup_mount(struct autofs_point *ap, struct map_source *map, const char
 	return status;
 }
 
+static int lookup_amd_instance(struct autofs_point *ap,
+			       struct map_source *map,
+			       const char *name, int name_len)
+{
+	struct map_source *instance;
+	struct amd_entry *entry;
+	const char *argv[2];
+	const char **pargv = NULL;
+	int argc = 0;
+	struct mapent *me;
+	char *m_key;
+
+	me = cache_lookup_distinct(map->mc, name);
+	if (!me || !me->multi) {
+		error(ap->logopt, "expected multi mount entry not found");
+		return NSS_STATUS_UNKNOWN;
+	}
+
+	m_key = malloc(strlen(ap->path) + strlen(me->multi->key) + 1);
+	if (!m_key) {
+		error(ap->logopt,
+		     "failed to allocate storage for search key");
+		return NSS_STATUS_UNKNOWN;
+	}
+
+	strcpy(m_key, ap->path);
+	strcat(m_key, "/");
+	strcat(m_key, me->multi->key);
+	entry = master_find_amdmount(ap, m_key);
+	free(m_key);
+
+	if (!entry) {
+		error(ap->logopt, "expected amd mount entry not found");
+		return NSS_STATUS_UNKNOWN;
+	}
+
+	if (strcmp(entry->type, "host")) {
+		error(ap->logopt, "unexpected map type %s", entry->type);
+		return NSS_STATUS_UNKNOWN;
+	}
+
+	if (entry->opts && *entry->opts) {
+		argv[0] = entry->opts;
+		argv[1] = NULL;
+		pargv = argv;
+		argc = 1;
+	}
+
+	instance = master_find_source_instance(map, "hosts", "sun", argc, pargv);
+	/* If this is an nss map instance it may have an amd host map sub instance */
+	if (!instance && map->instance) {
+		struct map_source *next = map->instance;
+		while (next) {
+			instance = master_find_source_instance(next,
+						"hosts", "sun", argc, pargv);
+			if (instance)
+				break;
+			next = next->next;
+		}
+	}
+	if (!instance) {
+		error(ap->logopt, "expected hosts map instance not found");
+		return NSS_STATUS_UNKNOWN;
+	}
+
+	return do_lookup_mount(ap, instance, name, name_len);
+}
+
 static int lookup_name_file_source_instance(struct autofs_point *ap, struct map_source *map, const char *name, int name_len)
 {
 	struct map_source *instance;
@@ -755,6 +879,9 @@ static int lookup_name_file_source_instance(struct autofs_point *ap, struct map_
 	struct stat st;
 	char *type, *format;
 
+	if (*name == '/' && map->flags & MAP_FLAG_FORMAT_AMD)
+		return lookup_amd_instance(ap, map, name, name_len);
+
 	if (stat(map->argv[0], &st) == -1) {
 		debug(ap->logopt, "file map not found");
 		return NSS_STATUS_NOTFOUND;
@@ -790,6 +917,9 @@ static int lookup_name_source_instance(struct autofs_point *ap, struct map_sourc
 	const char *format;
 	time_t age = time(NULL);
 
+	if (*name == '/' && map->flags & MAP_FLAG_FORMAT_AMD)
+		return lookup_amd_instance(ap, map, name, name_len);
+
 	format = map->format;
 
 	instance = master_find_source_instance(map, type, format, 0, NULL);
@@ -818,6 +948,9 @@ static int do_name_lookup_mount(struct autofs_point *ap,
 		return NSS_STATUS_UNKNOWN;
 	}
 
+	if (*name == '/' && map->flags & MAP_FLAG_FORMAT_AMD)
+		return lookup_amd_instance(ap, map, name, name_len);
+
 	/*
 	 * This is only called when map->type != NULL.
 	 * We only need to look for a map if source type is
@@ -829,7 +962,7 @@ static int do_name_lookup_mount(struct autofs_point *ap,
 	if (map->argv[0][0] == '/')
 		return do_lookup_mount(ap, map, name, name_len);
 
-	path = find_map_path(map);
+	path = find_map_path(ap, map);
 	if (!path)
 		return NSS_STATUS_UNKNOWN;
 
@@ -871,8 +1004,10 @@ static enum nsswitch_status lookup_map_name(struct nss_source *this,
 	}
 
 	this->source[4] = '\0';
+	tmap.flags = map->flags;
 	tmap.type = this->source;
 	tmap.format = map->format;
+	tmap.name = map->name;
 	tmap.mc = map->mc;
 	tmap.instance = map->instance;
 	tmap.exp_timeout = map->exp_timeout;
@@ -881,7 +1016,7 @@ static enum nsswitch_status lookup_map_name(struct nss_source *this,
 	tmap.argc = 0;
 	tmap.argv = NULL;
 
-	path = find_map_path(map);
+	path = find_map_path(ap, map);
 	if (!path)
 		return NSS_STATUS_UNKNOWN;
 
@@ -922,6 +1057,10 @@ static void update_negative_cache(struct autofs_point *ap, struct map_source *so
 	if (source && source->depth)
 		return;
 
+	/* Don't update the wildcard */
+	if (strlen(name) == 1 && *name == '*')
+		return;
+
 	/* Have we recorded the lookup fail for negative caching? */
 	me = lookup_source_mapent(ap, name, LKP_DISTINCT);
 	if (me)
@@ -1036,6 +1175,13 @@ int lookup_nss_mount(struct autofs_point *ap, struct map_source *source, const c
 		list_for_each(p, head) {
 			this = list_entry(p, struct nss_source, list);
 
+			if (map->flags & MAP_FLAG_FORMAT_AMD &&
+			    !strcmp(this->source, "sss")) {
+				warn(ap->logopt,
+				     "source sss is not available for amd maps.");
+				continue;
+			}
+
 			result = lookup_map_name(this, ap, map, name, name_len);
 
 			if (result == NSS_STATUS_UNKNOWN)
@@ -1058,7 +1204,6 @@ int lookup_nss_mount(struct autofs_point *ap, struct map_source *source, const c
 	}
 	if (ap->state != ST_INIT)
 		send_map_update_request(ap);
-	pthread_cleanup_pop(1);
 
 	/*
 	 * The last source lookup will return NSS_STATUS_NOTFOUND if the
@@ -1067,6 +1212,7 @@ int lookup_nss_mount(struct autofs_point *ap, struct map_source *source, const c
 	 */
 	if (result == NSS_STATUS_NOTFOUND || result == NSS_STATUS_UNAVAIL)
 		update_negative_cache(ap, source, name);
+	pthread_cleanup_pop(1);
 
 	return !result;
 }
@@ -1143,8 +1289,11 @@ void lookup_prune_one_cache(struct autofs_point *ap, struct mapent_cache *mc, ti
 			/*
 			 * Reset time of last fail for valid map entries to
 			 * force entry update and subsequent mount retry.
+			 * A map entry that's still invalid after a read
+			 * may have been created by a failed wildcard lookup
+			 * so reset the status on those too.
 			 */
-			if (me->mapent)
+			if (me->mapent || cache_lookup(mc, "*"))
 				me->status = 0;
 			me = cache_enumerate(mc, me);
 			continue;
