@@ -1,15 +1,19 @@
 #ifndef LOOKUP_LDAP_H
 #define LOOKUP_LDAP_H
 
+#include <ldap.h>
+
 #ifdef WITH_SASL
 #include <openssl/ssl.h>
 #include <openssl/evp.h>
 #include <openssl/err.h>
 #include <sasl/sasl.h>
-#include <libxml/tree.h>
 #include <krb5.h>
 #endif
 
+#include <libxml/tree.h>
+
+#include "list.h"
 #include "dclist.h"
 
 struct ldap_schema {
@@ -32,6 +36,7 @@ struct ldap_searchdn {
 
 struct lookup_context {
 	char *mapname;
+	unsigned int format;
 
 	char *server;
 	int port;
@@ -39,6 +44,8 @@ struct lookup_context {
 	char *qdn;
 	unsigned int timeout;
 	unsigned int network_timeout;
+	unsigned long timestamp;
+	unsigned int check_defaults;
 
 	/* LDAP version 2 or 3 */
 	int version;
@@ -76,9 +83,13 @@ struct lookup_context {
 	int          kinit_done;
 	int          kinit_successful;
 #ifdef WITH_SASL
+	/* Kerberos */
 	krb5_context krb5ctxt;
 	krb5_ccache  krb5_ccache;
 	sasl_conn_t  *sasl_conn;
+	/* SASL external */
+	char	     *extern_cert;
+	char	     *extern_key;
 #endif
 	/* keytab file name needs to be added */
 
@@ -86,7 +97,6 @@ struct lookup_context {
 };
 
 
-#ifdef WITH_SASL
 #define LDAP_AUTH_CONF_FILE "test"
 
 #define LDAP_TLS_DONT_USE	0
@@ -97,12 +107,16 @@ struct lookup_context {
 #define LDAP_AUTH_NOTREQUIRED	0x0001
 #define LDAP_AUTH_REQUIRED	0x0002
 #define LDAP_AUTH_AUTODETECT	0x0004
+#define LDAP_NEED_AUTH		(LDAP_AUTH_REQUIRED|LDAP_AUTH_AUTODETECT)
+
+#define LDAP_AUTH_USESIMPLE	0x0008
 
 /* lookup_ldap.c */
 LDAP *init_ldap_connection(unsigned logopt, const char *uri, struct lookup_context *ctxt);
 int unbind_ldap_connection(unsigned logopt, LDAP *ldap, struct lookup_context *ctxt);
 int authtype_requires_creds(const char *authtype);
 
+#ifdef WITH_SASL
 /* cyrus-sasl.c */
 int autofs_sasl_client_init(unsigned logopt);
 int autofs_sasl_init(unsigned logopt, LDAP *ldap, struct lookup_context *ctxt);
@@ -110,6 +124,8 @@ int autofs_sasl_bind(unsigned logopt, LDAP *ldap, struct lookup_context *ctxt);
 void autofs_sasl_unbind(struct lookup_context *ctxt);
 void autofs_sasl_dispose(struct lookup_context *ctxt);
 void autofs_sasl_done(void);
+/* cyrus-sasl-extern */
+int do_sasl_extern(LDAP *ldap, struct lookup_context *ctxt);
 #endif
 
 #endif
