@@ -120,7 +120,7 @@ void master_notify_state_change(struct master *, int);
 int master_mount_mounts(struct master *, time_t, int);
 int dump_map(struct master *, const char *, const char *);
 int master_show_mounts(struct master *);
-extern inline unsigned int master_get_logopt(void);
+unsigned int master_get_logopt(void);
 int master_list_empty(struct master *);
 int master_done(struct master *);
 int master_kill(struct master *);
