@@ -40,22 +40,6 @@ do { \
 		fatal(_alm_unlock); \
 } while (0)
 
-void dump_alarms(void)
-{
-	struct list_head *head;
-	struct list_head *p;
-
-	pthread_mutex_lock(&mutex);
-	head = &alarms;
-	list_for_each(p, head) {
-		struct alarm *this;
-
-		this = list_entry(p, struct alarm, list);
-		logmsg("alarm time = %d", this->time);
-	}
-	pthread_mutex_unlock(&mutex);
-}
-
 /* Insert alarm entry on ordered list. */
 int alarm_add(struct autofs_point *ap, time_t seconds)
 {
@@ -67,6 +51,9 @@ int alarm_add(struct autofs_point *ap, time_t seconds)
 	unsigned int empty = 1;
 	int status;
 
+	if (!seconds)
+		return 1;
+
 	new = malloc(sizeof(struct alarm));
 	if (!new)
 		return 0;
@@ -238,9 +225,10 @@ int alarm_start_handler(void)
 	}
 
 	status = pthread_create(&thid, pattrs, alarm_handler, NULL);
-	if (status)
-	 	return 0;
 
-	return 1;
+	if (pattrs)
+		pthread_attr_destroy(pattrs);
+
+	return !status;
 }
 
