@@ -20,7 +20,6 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <sys/param.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 
 #include "automount.h"
@@ -204,7 +203,7 @@ int cache_push_mapent(struct mapent *me, char *mapent)
 
 	s->mapent = me->mapent;
 	s->age = me->age;
-	me->mapent = mapent;
+	me->mapent = new;
 
 	if (me->stack)
 		s->next = me->stack;
@@ -362,24 +361,6 @@ struct mapent_cache *cache_init_null_cache(struct master *master)
 	return mc;
 }
 
-static u_int32_t hash(const char *key, unsigned int size)
-{
-	u_int32_t hashval;
-	char *s = (char *) key;
-
-	for (hashval = 0; *s != '\0';) {
-		hashval += (unsigned char) *s++;
-		hashval += (hashval << 10);
-		hashval ^= (hashval >> 6);
-	}
-
-	hashval += (hashval << 3);
-	hashval ^= (hashval >> 11);
-	hashval += (hashval << 15);
-
-	return hashval % size;
-}
-
 static u_int32_t ino_hash(dev_t dev, ino_t ino, unsigned int size)
 {
 	u_int32_t hashval;
@@ -585,7 +566,9 @@ struct mapent *cache_lookup_offset(const char *prefix, const char *offset, int s
 }
 
 /* cache must be read locked by caller */
-struct mapent *cache_partial_match(struct mapent_cache *mc, const char *prefix)
+static struct mapent *__cache_partial_match(struct mapent_cache *mc,
+					    const char *prefix,
+					    unsigned int type)
 {
 	struct mapent *me = NULL;
 	size_t len = strlen(prefix);
@@ -597,20 +580,46 @@ struct mapent *cache_partial_match(struct mapent_cache *mc, const char *prefix)
 			continue;
 
 		if (len < strlen(me->key) &&
-		    (strncmp(prefix, me->key, len) == 0) && me->key[len] == '/')
-			return me;
+		    (strncmp(prefix, me->key, len) == 0) &&
+		     me->key[len] == '/') {
+			if (type == LKP_NORMAL)
+				return me;
+			if (type == LKP_WILD &&
+			    me->key[len] != '\0' &&
+			    me->key[len + 1] == '*')
+				return me;
+		}
 
 		me = me->next;
 		while (me != NULL) {
 			if (len < strlen(me->key) &&
-			    strncmp(prefix, me->key, len) == 0 && me->key[len] == '/')
-				return me;
+			    (strncmp(prefix, me->key, len) == 0 &&
+			    me->key[len] == '/')) {
+				if (type == LKP_NORMAL)
+					return me;
+				if (type == LKP_WILD &&
+				    me->key[len] != '\0' &&
+				    me->key[len + 1] == '*')
+					return me;
+			}
 			me = me->next;
 		}
 	}
 	return NULL;
 }
 
+/* cache must be read locked by caller */
+struct mapent *cache_partial_match(struct mapent_cache *mc, const char *prefix)
+{
+	return __cache_partial_match(mc, prefix, LKP_NORMAL);
+}
+
+/* cache must be read locked by caller */
+struct mapent *cache_partial_match_wild(struct mapent_cache *mc, const char *prefix)
+{
+	return __cache_partial_match(mc, prefix, LKP_WILD);
+}
+
 /* cache must be write locked by caller */
 int cache_add(struct mapent_cache *mc, struct map_source *ms, const char *key, const char *mapent, time_t age)
 {
