@@ -20,6 +20,7 @@
 #include <string.h>
 #include <sys/utsname.h>
 #include <sys/stat.h>
+#include <stdarg.h>
 
 #include "config.h"
 #include "list.h"
@@ -50,6 +51,7 @@
 #define NAME_NEGATIVE_TIMEOUT		"negative_timeout"
 #define NAME_BROWSE_MODE		"browse_mode"
 #define NAME_LOGGING			"logging"
+#define NAME_FORCE_STD_PROG_MAP_ENV	"force_standard_program_map_env"
 
 #define NAME_LDAP_URI			"ldap_uri"
 #define NAME_LDAP_TIMEOUT		"ldap_timeout"
@@ -171,6 +173,19 @@ static int conf_update(const char *, const char *, const char *, unsigned long);
 static void conf_delete(const char *, const char *);
 static struct conf_option *conf_lookup(const char *, const char *);
 
+static void defaults_mutex_lock(void)
+{
+	int status = pthread_mutex_lock(&conf_mutex);
+	if (status)
+		fatal(status);
+}
+
+static void defaults_mutex_unlock(void)
+{
+	int status = pthread_mutex_unlock(&conf_mutex);
+	if (status)
+		fatal(status);
+}
 
 static void message(unsigned int to_syslog, const char *msg, ...)
 {
@@ -253,9 +268,9 @@ static void __conf_release(void)
 
 void defaults_conf_release(void)
 {
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	__conf_release();
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 	return;
 }
 
@@ -727,11 +742,11 @@ static unsigned int conf_section_exists(const char *section)
 		return 0;
 
 	ret = 0;
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(section, section);
 	if (co)
 		ret = 1;
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 
 	return ret;
 }
@@ -834,7 +849,7 @@ static int parse_line(char *line, char **sec, char **res, char **value)
 
 static int read_config(unsigned int to_syslog, FILE *f, const char *name)
 {
-	char buf[MAX_LINE_LEN];
+	char buf[MAX_LINE_LEN + 2];
 	char secbuf[MAX_SECTION_NAME];
 	char *new_sec;
 	char *res;
@@ -842,6 +857,12 @@ static int read_config(unsigned int to_syslog, FILE *f, const char *name)
 	new_sec = NULL;
 	while ((res = fgets(buf, MAX_LINE_LEN, f))) {
 		char *sec, *key, *value;
+
+		if (strlen(res) > MAX_LINE_LEN) {
+			message(to_syslog, "%s was truncated, ignored", res);
+			continue;
+		}
+
 		sec = key = value = NULL;
 		if (!parse_line(res, &sec, &key, &value))
 			continue;
@@ -1049,7 +1070,9 @@ unsigned int defaults_read_config(unsigned int to_syslog)
 
 	ret = 1;
 
-	pthread_mutex_lock(&conf_mutex);
+	conf = oldconf = NULL;
+
+	defaults_mutex_lock();
 	if (!config) {
 		if (conf_init()) {
 			message(to_syslog, "failed to init config");
@@ -1075,15 +1098,11 @@ unsigned int defaults_read_config(unsigned int to_syslog)
 	    stb.st_mtime <= config->modified &&
 	    (oldstat = fstat(fileno(oldconf), &oldstb) == -1) &&
 	    oldstb.st_mtime <= config->modified) {
-		fclose(conf);
-		fclose(oldconf);
 		goto out;
 	}
 
 	if (conf || oldconf) {
 		if (!reset_defaults(to_syslog)) {
-			fclose(conf);
-			fclose(oldconf);
 			ret = 0;
 			goto out;
 		}
@@ -1101,10 +1120,8 @@ unsigned int defaults_read_config(unsigned int to_syslog)
 		}
 	}
 
-	if (conf) {
+	if (conf)
 		read_config(to_syslog, conf, DEFAULT_CONFIG_FILE);
-		fclose(conf);
-	}
 
 	/*
 	 * Read the old config file and override the installed
@@ -1125,7 +1142,6 @@ unsigned int defaults_read_config(unsigned int to_syslog)
 			clean_ldap_multi_option(NAME_LDAP_URI);
 
 		read_config(to_syslog, oldconf, OLD_CONFIG_FILE);
-		fclose(oldconf);
 
 		if (ldap_search_base) {
 			co = conf_lookup(sec, NAME_SEARCH_BASE);
@@ -1144,7 +1160,11 @@ unsigned int defaults_read_config(unsigned int to_syslog)
 		}
 	}
 out:
-	pthread_mutex_unlock(&conf_mutex);
+	if (conf)
+		fclose(conf);
+	if (oldconf)
+		fclose(oldconf);
+	defaults_mutex_unlock();
 	return ret;
 }
 
@@ -1153,11 +1173,11 @@ static char *conf_get_string(const char *section, const char *name)
 	struct conf_option *co;
 	char *val = NULL;
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(section, name);
 	if (co && co->value)
 		val = strdup(co->value);
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 	return val;
 }
 
@@ -1166,11 +1186,11 @@ static long conf_get_number(const char *section, const char *name)
 	struct conf_option *co;
 	long val = -1;
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(section, name);
 	if (co && co->value)
 		val = atol(co->value);
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 	return val;
 }
 
@@ -1179,7 +1199,7 @@ static int conf_get_yesno(const char *section, const char *name)
 	struct conf_option *co;
 	int val = -1;
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(section, name);
 	if (co && co->value) {
 		if (isdigit(*co->value))
@@ -1189,7 +1209,7 @@ static int conf_get_yesno(const char *section, const char *name)
 		else if (!strcasecmp(co->value, "no"))
 			val = 0;
 	}
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 	return val;
 }
 
@@ -1266,10 +1286,10 @@ struct list_head *defaults_get_uris(void)
 		return NULL;
 	}
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(autofs_gbl_sec, NAME_LDAP_URI);
 	if (!co) {
-		pthread_mutex_unlock(&conf_mutex);
+		defaults_mutex_unlock();
 		free(list);
 		return NULL;
 	}
@@ -1280,7 +1300,7 @@ struct list_head *defaults_get_uris(void)
 				add_uris(co->value, list);
 		co = co->next;
 	}
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 
 	if (list_empty(list)) {
 		free(list);
@@ -1392,10 +1412,10 @@ struct ldap_searchdn *defaults_get_searchdns(void)
 	if (!defaults_read_config(0))
 		return NULL;
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(autofs_gbl_sec, NAME_SEARCH_BASE);
 	if (!co) {
-		pthread_mutex_unlock(&conf_mutex);
+		defaults_mutex_unlock();
 		return NULL;
 	}
 
@@ -1411,7 +1431,7 @@ struct ldap_searchdn *defaults_get_searchdns(void)
 
 		new = alloc_searchdn(co->value);
 		if (!new) {
-			pthread_mutex_unlock(&conf_mutex);
+			defaults_mutex_unlock();
 			defaults_free_searchdns(sdn);
 			return NULL;
 		}
@@ -1428,7 +1448,7 @@ struct ldap_searchdn *defaults_get_searchdns(void)
 
 		co = co->next;
 	}
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 
 	return sdn;
 }
@@ -1506,9 +1526,9 @@ int defaults_master_set(void)
 {
 	struct conf_option *co;
 
-	pthread_mutex_lock(&conf_mutex);
+	defaults_mutex_lock();
 	co = conf_lookup(autofs_gbl_sec, NAME_MASTER_MAP);
-	pthread_mutex_unlock(&conf_mutex);
+	defaults_mutex_unlock();
 	if (co)
 		return 1;
 	return 0;
@@ -1571,6 +1591,17 @@ unsigned int defaults_get_logging(void)
 	return logging;
 }
 
+unsigned int defaults_force_std_prog_map_env(void)
+{
+	int res;
+
+	res = conf_get_yesno(autofs_gbl_sec, NAME_FORCE_STD_PROG_MAP_ENV);
+	if (res < 0)
+		res = atoi(DEFAULT_FORCE_STD_PROG_MAP_ENV);
+
+	return res;
+}
+
 unsigned int defaults_get_ldap_timeout(void)
 {
 	int res;
@@ -1789,6 +1820,7 @@ unsigned int conf_amd_get_log_options(void)
 			if (log_level < LOG_CRIT)
 				log_level = LOG_CRIT;
 		}
+		free(tmp);
 	}
 
 	if (log_level == -1)
