@@ -26,6 +26,7 @@
 #include <sys/vfs.h>
 #include <pwd.h>
 #include <grp.h>
+#include <libgen.h>
 
 #include "automount.h"
 
@@ -47,6 +48,18 @@ static const char mnt_name_template[]      = "automount(pid%u)";
 static struct kernel_mod_version kver = {0, 0};
 static const char kver_options_template[]  = "fd=%d,pgrp=%u,minproto=3,maxproto=5";
 
+#define EXT_MOUNTS_HASH_SIZE    50
+
+struct ext_mount {
+	char *mountpoint;
+	unsigned int umount;
+	struct list_head mount;
+	struct list_head mounts;
+};
+static struct list_head ext_mounts_hash[EXT_MOUNTS_HASH_SIZE];
+static unsigned int ext_mounts_hash_init_done = 0;
+static pthread_mutex_t ext_mount_hash_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 unsigned int linux_version_code(void)
 {
 	struct utsname my_utsname;
@@ -422,6 +435,163 @@ struct substvar *removestdenv(struct substvar *sv, const char *prefix)
 	return list;
 }
 
+void add_std_amd_vars(struct substvar *sv)
+{
+	char *tmp;
+
+	tmp = conf_amd_get_arch();
+	if (tmp) {
+		macro_global_addvar("arch", 4, tmp);
+		free(tmp);
+	}
+
+	tmp = conf_amd_get_karch();
+	if (tmp) {
+		macro_global_addvar("karch", 5, tmp);
+		free(tmp);
+	}
+
+	tmp = conf_amd_get_os();
+	if (tmp) {
+		macro_global_addvar("os", 2, tmp);
+		free(tmp);
+	}
+
+	tmp = conf_amd_get_full_os();
+	if (tmp) {
+		macro_global_addvar("full_os", 7, tmp);
+		free(tmp);
+	}
+
+	tmp = conf_amd_get_os_ver();
+	if (tmp) {
+		macro_global_addvar("osver", 5, tmp);
+		free(tmp);
+	}
+
+	tmp = conf_amd_get_vendor();
+	if (tmp) {
+		macro_global_addvar("vendor", 6, tmp);
+		free(tmp);
+	}
+
+	/* Umm ... HP_UX cluster name, probably not used */
+	tmp = conf_amd_get_cluster();
+	if (tmp) {
+		macro_global_addvar("cluster", 7, tmp);
+		free(tmp);
+	} else {
+		const struct substvar *v = macro_findvar(sv, "domain", 4);
+		if (v && *v->val) {
+			tmp = strdup(v->val);
+			if (tmp)
+				macro_global_addvar("cluster", 7, tmp);
+		}
+	}
+
+	tmp = conf_amd_get_auto_dir();
+	if (tmp) {
+		macro_global_addvar("autodir", 7, tmp);
+		free(tmp);
+	}
+
+	return;
+}
+
+void remove_std_amd_vars(void)
+{
+	macro_global_removevar("autodir", 7);
+	macro_global_removevar("cluster", 7);
+	macro_global_removevar("vendor", 6);
+	macro_global_removevar("osver", 5);
+	macro_global_removevar("full_os", 7);
+	macro_global_removevar("os", 2);
+	macro_global_removevar("karch", 5);
+	macro_global_removevar("arch", 4);
+	return;
+ }
+
+struct amd_entry *new_amd_entry(const struct substvar *sv)
+{
+	struct amd_entry *new;
+	const struct substvar *v;
+	char *path;
+
+	v = macro_findvar(sv, "path", 4);
+	if (!v)
+		return NULL;
+
+	path = strdup(v->val);
+	if (!path)
+		return NULL;
+
+	new = malloc(sizeof(struct amd_entry));
+	if (!new) {
+		free(path);
+		return NULL;
+	}
+
+	memset(new, 0, sizeof(*new));
+	new->path = path;
+	INIT_LIST_HEAD(&new->list);
+	INIT_LIST_HEAD(&new->entries);
+	INIT_LIST_HEAD(&new->ext_mount);
+
+	return new;
+}
+
+void clear_amd_entry(struct amd_entry *entry)
+{
+	if (!entry)
+		return;
+	if (entry->path)
+		free(entry->path);
+	if (entry->map_type)
+		free(entry->map_type);
+	if (entry->pref)
+		free(entry->pref);
+	if (entry->fs)
+		free(entry->fs);
+	if (entry->rhost)
+		free(entry->rhost);
+	if (entry->rfs)
+		free(entry->rfs);
+	if (entry->opts)
+		free(entry->opts);
+	if (entry->addopts)
+		free(entry->addopts);
+	if (entry->remopts)
+		free(entry->remopts);
+	if (entry->sublink)
+		free(entry->sublink);
+	if (entry->selector)
+		free_selector(entry->selector);
+	return;
+}
+
+void free_amd_entry(struct amd_entry *entry)
+{
+	clear_amd_entry(entry);
+	free(entry);
+	return;
+}
+
+void free_amd_entry_list(struct list_head *entries)
+{
+	if (!list_empty(entries)) {
+		struct list_head *head = entries;
+		struct amd_entry *this;
+		struct list_head *p;
+
+		p = head->next;
+		while (p != head) {
+			this = list_entry(p, struct amd_entry, list);
+			p = p->next;
+			free_amd_entry(this);
+		}
+	}
+}
+
 /*
  * Make common autofs mount options string
  */
@@ -490,6 +660,129 @@ char *make_mnt_name_string(char *path)
 	return mnt_name;
 }
 
+static void ext_mounts_hash_init(void)
+{
+	int i;
+	for (i = 0; i < EXT_MOUNTS_HASH_SIZE; i++)
+		INIT_LIST_HEAD(&ext_mounts_hash[i]);
+	ext_mounts_hash_init_done = 1;
+}
+
+static struct ext_mount *ext_mount_lookup(const char *mountpoint)
+{
+	u_int32_t hval = hash(mountpoint, EXT_MOUNTS_HASH_SIZE);
+	struct list_head *p, *head;
+
+	if (!ext_mounts_hash_init_done)
+		ext_mounts_hash_init();
+
+	if (list_empty(&ext_mounts_hash[hval]))
+		return NULL;
+
+	head = &ext_mounts_hash[hval];
+	list_for_each(p, head) {
+		struct ext_mount *this = list_entry(p, struct ext_mount, mount);
+		if (!strcmp(this->mountpoint, mountpoint))
+			return this;
+	}
+	return NULL;
+}
+
+int ext_mount_add(struct list_head *entry, const char *path, unsigned int umount)
+{
+	struct ext_mount *em;
+	char *auto_dir;
+	u_int32_t hval;
+	int ret = 0;
+
+	/* Not a mount in the external mount directory */
+	auto_dir = conf_amd_get_auto_dir();
+	if (strncmp(path, auto_dir, strlen(auto_dir))) {
+		free(auto_dir);
+		return 0;
+	}
+	free(auto_dir);
+
+	pthread_mutex_lock(&ext_mount_hash_mutex);
+
+	em = ext_mount_lookup(path);
+	if (em) {
+		struct list_head *p, *head;
+		head = &em->mounts;
+		list_for_each(p, head) {
+			if (p == entry)
+				goto done;
+		}
+		list_add_tail(entry, &em->mounts);
+		ret = 1;
+		goto done;
+	}
+
+	em = malloc(sizeof(struct ext_mount));
+	if (!em) {
+		ret = -1;
+		goto done;
+	}
+
+	em->mountpoint = strdup(path);
+	if (!em->mountpoint) {
+		free(em);
+		ret = -1;
+		goto done;
+	}
+	em->umount = umount;
+	INIT_LIST_HEAD(&em->mount);
+	INIT_LIST_HEAD(&em->mounts);
+
+	hval = hash(path, EXT_MOUNTS_HASH_SIZE);
+	list_add_tail(&em->mount, &ext_mounts_hash[hval]);
+
+	list_add_tail(entry, &em->mounts);
+
+	ret = 1;
+done:
+	pthread_mutex_unlock(&ext_mount_hash_mutex);
+	return ret;
+}
+
+int ext_mount_remove(struct list_head *entry, const char *path)
+{
+	struct ext_mount *em;
+	char *auto_dir;
+	int ret = 0;
+
+	/* Not a mount in the external mount directory */
+	auto_dir = conf_amd_get_auto_dir();
+	if (strncmp(path, auto_dir, strlen(auto_dir))) {
+		free(auto_dir);
+		return 0;
+	}
+	free(auto_dir);
+
+	pthread_mutex_lock(&ext_mount_hash_mutex);
+
+	em = ext_mount_lookup(path);
+	if (!em)
+		goto done;
+
+	list_del_init(entry);
+
+	if (!list_empty(&em->mounts))
+		goto done;
+	else {
+		list_del_init(&em->mount);
+		if (em->umount)
+			ret = 1;
+		if (list_empty(&em->mount)) {
+			free(em->mountpoint);
+			free(em);
+		}
+	}
+done:
+	pthread_mutex_unlock(&ext_mount_hash_mutex);
+	return ret;
+}
+
 /*
  * Get list of mounts under path in longest->shortest order
  */
