@@ -19,10 +19,6 @@
 
 #include <stdio.h>
 #include <malloc.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <syslog.h>
 #include <string.h>
 #include <sys/param.h>
 #include <sys/types.h>
@@ -38,8 +34,6 @@
 
 int mount_version = AUTOFS_MOUNT_VERSION;	/* Required by protocol */
 
-extern struct autofs_point ap;
-
 int swapCD(const char *device, const char *slotName);
 
 int mount_init(void **context)
@@ -47,79 +41,96 @@ int mount_init(void **context)
 	return 0;
 }
 
-int mount_mount(const char *root, const char *name, int name_len,
+int mount_mount(struct autofs_point *ap, const char *root, const char *name, int name_len,
 		const char *what, const char *fstype, const char *options, void *context)
 {
-	char *fullpath;
+	char fullpath[PATH_MAX];
+	char buf[MAX_ERR_BUF];
 	int err;
-	int status;
+	int len, status, existed = 1;
+
+	if (ap->flags & MOUNT_FLAG_REMOUNT)
+		return 0;
 
 	fstype = "iso9660";
 
-	fullpath = alloca(strlen(root) + name_len + 2);
-	if (!fullpath) {
-		error(MODPREFIX "alloca: %m");
-		return 1;
+	/* Root offset of multi-mount */
+	len = strlen(root);
+	if (root[len - 1] == '/') {
+		len = snprintf(fullpath, len, "%s", root);
+	} else if (*name == '/') {
+		/*
+		 * Direct or offset mount, name is absolute path so
+		 * don't use root (but with move mount changes root
+		 * is now the same as name).
+		 */
+		len = sprintf(fullpath, "%s", root);
+	} else {
+		len = sprintf(fullpath, "%s/%s", root, name);
 	}
+	fullpath[len] = '\0';
 
-	if (name_len)
-		sprintf(fullpath, "%s/%s", root, name);
-	else
-		sprintf(fullpath, "%s", root);
+	debug(ap->logopt, MODPREFIX "calling umount %s", what);
 
-	debug(MODPREFIX "calling umount %s", what);
-
-	wait_for_lock();
-	err = spawnl(LOG_DEBUG, MOUNTED_LOCK,
-		     PATH_UMOUNT, PATH_UMOUNT, what, NULL);
-	unlink(AUTOFS_LOCK);
+	err = spawn_umount(ap->logopt, what, NULL);
 	if (err) {
-		error(MODPREFIX "umount of %s failed (all may be unmounted)",
+		error(ap->logopt,
+		      MODPREFIX
+		      "umount of %s failed (all may be unmounted)",
 		      what);
 	}
 
-	debug(MODPREFIX "calling mkdir_path %s", fullpath);
+	debug(ap->logopt, MODPREFIX "calling mkdir_path %s", fullpath);
 
-	if ((status = mkdir_path(fullpath, 0555)) && errno != EEXIST) {
-		error(MODPREFIX "mkdir_path %s failed: %m", fullpath);
+	status = mkdir_path(fullpath, 0555);
+	if (status && errno != EEXIST) {
+		char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
+		error(ap->logopt,
+		      MODPREFIX "mkdir_path %s failed: %s", fullpath, estr);
 		return 1;
 	}
 
-	debug(MODPREFIX "Swapping CD to slot %s", name);
+	if (!status)
+		existed = 0;
+
+	debug(ap->logopt, MODPREFIX "Swapping CD to slot %s", name);
 
 	err = swapCD(what, name);
 	if (err) {
-		error(MODPREFIX "failed to swap CD to slot %s", name);
+		error(ap->logopt,
+		      MODPREFIX "failed to swap CD to slot %s", name);
 		return 1;
 	}
 
-	wait_for_lock();
-	if (options) {
-		debug(MODPREFIX "calling mount -t %s " SLOPPY "-o %s %s %s",
-		    fstype, options, what, fullpath);
+	if (options && options[0]) {
+		debug(ap->logopt, MODPREFIX
+		      "calling mount -t %s -o %s %s %s",
+		      fstype, options, what, fullpath);
 
-		err = spawnl(LOG_DEBUG, MOUNTED_LOCK,
-			     PATH_MOUNT, PATH_MOUNT, "-t", fstype,
-			     SLOPPYOPT "-o", options, what, fullpath, NULL);
+		err = spawn_mount(ap->logopt, "-t", fstype,
+			          "-o", options, what, fullpath, NULL);
 	} else {
-		debug(MODPREFIX "calling mount -t %s %s %s",
-			  fstype, what, fullpath);
+		debug(ap->logopt,
+		      MODPREFIX "calling mount -t %s %s %s",
+		      fstype, what, fullpath);
 
-		err = spawnl(LOG_DEBUG, MOUNTED_LOCK, PATH_MOUNT, PATH_MOUNT,
-			     "-t", fstype, what, fullpath, NULL);
+		err = spawn_mount(ap->logopt, "-t", fstype, what, fullpath, NULL);
 	}
-	unlink(AUTOFS_LOCK);
 
 	if (err) {
-		if (!ap.ghost && name_len)
-			rmdir_path(name);
+		info(ap->logopt, MODPREFIX "failed to mount %s (type %s) on %s",
+		    what, fstype, fullpath);
+
+		if (ap->type != LKP_INDIRECT)
+			return 1;
+
+		if ((!(ap->flags & MOUNT_FLAG_GHOST) && name_len) || !existed)
+			rmdir_path(ap, fullpath, ap->dev);
 
-		error(MODPREFIX "failed to mount %s (type %s) on %s",
-		      what, fstype, fullpath);
 		return 1;
 	} else {
-		debug(MODPREFIX "mounted %s type %s on %s",
-			  what, fstype, fullpath);
+		debug(ap->logopt, MODPREFIX "mounted %s type %s on %s",
+		    what, fstype, fullpath);
 		return 0;
 	}
 }
@@ -139,9 +150,9 @@ int swapCD(const char *device, const char *slotName)
 	slot = atoi(slotName) - 1;
 
 	/* open device */
-	fd = open(device, O_RDONLY | O_NONBLOCK);
+	fd = open_fd(device, O_RDONLY | O_NONBLOCK);
 	if (fd < 0) {
-		error(MODPREFIX "Opening device %s failed : %s",
+		logerr(MODPREFIX "Opening device %s failed : %s",
 		      device, strerror(errno));
 		return 1;
 	}
@@ -149,23 +160,25 @@ int swapCD(const char *device, const char *slotName)
 	/* Check CD player status */
 	total_slots_available = ioctl(fd, CDROM_CHANGER_NSLOTS);
 	if (total_slots_available <= 1) {
-		error(MODPREFIX
-		      "Device %s is not an ATAPI compliant CD changer.\n",
+		logerr(MODPREFIX
+		      "Device %s is not an ATAPI compliant CD changer.",
 		      device);
+		close(fd);
 		return 1;
 	}
 
 	/* load */
 	slot = ioctl(fd, CDROM_SELECT_DISC, slot);
 	if (slot < 0) {
-		error(MODPREFIX "CDROM_SELECT_DISC failed");
+		logerr(MODPREFIX "CDROM_SELECT_DISC failed");
+		close(fd);
 		return 1;
 	}
 
 	/* close device */
 	status = close(fd);
 	if (status != 0) {
-		error(MODPREFIX "close failed for `%s': %s\n",
+		logerr(MODPREFIX "close failed for `%s': %s",
 		      device, strerror(errno));
 		return 1;
 	}
